---
title: "Mutation and update cache"
---

import GithubLink from "../../src/GithubLink.js";
import YoutubeEmbed from "../../src/YoutubeEmbed.js";

<YoutubeEmbed link="https://www.youtube.com/embed/8Z9clUUg5ls" />

Now let's do the integration part. Open `src/components/Todo/TodoItem.js` and add the following code to define the delete mutation

<GithubLink link="https://github.com/hasura/graphql-engine/blob/master/community/learn/graphql-tutorials/tutorials/react-apollo/app-final/src/components/Todo/TodoItem.js" text="src/components/Todo/TodoItem.js" />

```javascript
const TodoItem = ({index, todo, client}) => {

+  const REMOVE_TODO = gql`
+    mutation removeTodo ($id: Int!) {
+      delete_todos(where: {id: {_eq: $id}}) {
+        affected_rows
+      }
+    }
+  `;
```

We have a function defined to handle the button click to remove a todo. Let's update the function to add `client.mutate` code.

```javascript
  const removeTodo = (e) => {
    e.preventDefault();
    e.stopPropagation();
+   client.mutate({
+     mutation: REMOVE_TODO,
+     variables: {id: todo.id},
+     optimisticResponse: {},
+     update: (cache) => {
+       const existingTodos = cache.readQuery({ query: GET_MY_TODOS });
+       const newTodos = existingTodos.todos.filter(t => (t.id !== todo.id));
+       cache.writeQuery({
+         query: GET_MY_TODOS,
+         data: {todos: newTodos}
+       });
+     }
+   });
  };
```